# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugin import LinterWarning
from troubadix.plugins.illegal_characters import CheckIllegalCharacters

from . import PluginTestCase


class CheckIllegalCharactersTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"cvss_base", value:"4.0");\n'
            'script_tag(name:"summary", value:"Foo Bar.");\n'
            'script_tag(name:"solution_type", value:"VendorFix");\n'
            'script_tag(name:"solution", value:"meh");\n'
        )

        results = list(
            CheckIllegalCharacters.run(
                path,
                content,
                tag_pattern=self.tag_pattern,
                special_tag_pattern=self.special_tag_pattern,
            )
        )
        self.assertEqual(len(results), 0)

    def test_illegal_chars_in_various_tags(self):
        tags = [
            "summary",
            "impact",
            "affected",
            "insight",
            "vuldetect",
            "solution",
        ]
        path = Path("tests/file.nasl")
        for tag in tags:
            content = (
                'script_tag(name:"cvss_base", value:"4.0");\n'
                f'script_tag(name:"{tag}", value:"Foo|Bar;Baz=Bad.");\n'
                'script_tag(name:"solution_type", value:"VendorFix");\n'
                'script_tag(name:"solution", value:"meh");\n'
            )

            expected_content = (
                'script_tag(name:"cvss_base", value:"4.0");\n'
                f'script_tag(name:"{tag}", value:"Foo Bar Baz Bad.");\n'
                'script_tag(name:"solution_type", value:"VendorFix");\n'
                'script_tag(name:"solution", value:"meh");\n'
            )

            results = list(
                CheckIllegalCharacters.run(
                    path,
                    content,
                    tag_pattern=self.tag_pattern,
                    special_tag_pattern=self.special_tag_pattern,
                )
            )
            self.assertEqual(len(results), 1)
            self.assertIsInstance(results[0], LinterWarning)
            self.assertEqual(
                results[0].message,
                f'Found illegal character in script_tag(name:"{tag}", '
                'value:"Foo|Bar;Baz=Bad.");',
            )
            self.assertEqual(
                path.read_text(encoding="latin1"),
                expected_content,
            )

        if path.exists():
            path.unlink()
